"use strict";(self.webpackChunkmlflow_oidc_auth_front=self.webpackChunkmlflow_oidc_auth_front||[]).push([["polyfills"],{3834:(fe,ae,Xe)=>{const Re=function(f,...k){if(Re.translate){const x=Re.translate(f,k);f=x[0],k=x[1]}let _=J(f[0],f.raw[0]);for(let x=1;x<f.length;x++)_+=k[x-1]+J(f[x],f.raw[x]);return _},we=":";function J(f,k){return k.charAt(0)===we?f.substring(function U(f,k){for(let _=1,x=1;_<f.length;_++,x++)if("\\"===k[x])x++;else if(":"===f[_])return _;throw new Error(`Unterminated $localize metadata block in "${k}".`)}(f,k)+1):f}
/**
         * @license Angular v19.2.7
         * (c) 2010-2025 Google LLC. https://angular.io/
         * License: MIT
         */globalThis.$localize=Re,Xe(6935)},6935:()=>{
/**
         * @license Angular v<unknown>
         * (c) 2010-2024 Google LLC. https://angular.io/
         * License: MIT
         */
const fe=globalThis;function ae(e){return(fe.__Zone_symbol_prefix||"__zone_symbol__")+e}const De=Object.getOwnPropertyDescriptor,Ne=Object.defineProperty,Ue=Object.getPrototypeOf,$e=Object.create,Ye=Array.prototype.slice,Ge="addEventListener",Le="removeEventListener",Me=ae(Ge),Ve=ae(Le),pe="true",_e="false",Se=ae("");function Ae(e,r){return Zone.current.wrap(e,r)}function Fe(e,r,c,t,i){return Zone.current.scheduleMacroTask(e,r,c,t,i)}const U=ae,Re=typeof window<"u",we=Re?window:void 0,J=Re&&we||globalThis;function f(e,r){for(let c=e.length-1;c>=0;c--)"function"==typeof e[c]&&(e[c]=Ae(e[c],r+"_"+c));return e}function _(e){return!e||!1!==e.writable&&!("function"==typeof e.get&&typeof e.set>"u")}const x=typeof WorkerGlobalScope<"u"&&self instanceof WorkerGlobalScope,$=!("nw"in J)&&typeof J.process<"u"&&"[object process]"===J.process.toString(),G=!$&&!x&&!(!Re||!we.HTMLElement),ke=typeof J.process<"u"&&"[object process]"===J.process.toString()&&!x&&!(!Re||!we.HTMLElement),te={},oe=U("enable_beforeunload"),Ee=function(e){if(!(e=e||J.event))return;let r=te[e.type];r||(r=te[e.type]=U("ON_PROPERTY"+e.type));const c=this||e.target||J,t=c[r];let i;return G&&c===we&&"error"===e.type?(i=t&&t.call(this,e.message,e.filename,e.lineno,e.colno,e.error),!0===i&&e.preventDefault()):(i=t&&t.apply(this,arguments),"beforeunload"===e.type&&J[oe]&&"string"==typeof i?e.returnValue=i:null!=i&&!i&&e.preventDefault()),i};function ze(e,r,c){let t=De(e,r);if(!t&&c&&De(c,r)&&(t={enumerable:!0,configurable:!0}),!t||!t.configurable)return;const i=U("on"+r+"patched");if(e.hasOwnProperty(i)&&e[i])return;delete t.writable,delete t.value;const u=t.get,g=t.set,p=r.slice(2);let b=te[p];b||(b=te[p]=U("ON_PROPERTY"+p)),t.set=function(I){let E=this;!E&&e===J&&(E=J),E&&("function"==typeof E[b]&&E.removeEventListener(p,Ee),g&&g.call(E,null),E[b]=I,"function"==typeof I&&E.addEventListener(p,Ee,!1))},t.get=function(){let I=this;if(!I&&e===J&&(I=J),!I)return null;const E=I[b];if(E)return E;if(u){let D=u.call(this);if(D)return t.set.call(this,D),"function"==typeof I.removeAttribute&&I.removeAttribute(r),D}return null},Ne(e,r,t),e[i]=!0}function Ze(e,r,c){if(r)for(let t=0;t<r.length;t++)ze(e,"on"+r[t],c);else{const t=[];for(const i in e)"on"==i.slice(0,2)&&t.push(i);for(let i=0;i<t.length;i++)ze(e,t[i],c)}}const he=U("originalInstance");function Oe(e){const r=J[e];if(!r)return;J[U(e)]=r,J[e]=function(){const i=f(arguments,e);switch(i.length){case 0:this[he]=new r;break;case 1:this[he]=new r(i[0]);break;case 2:this[he]=new r(i[0],i[1]);break;case 3:this[he]=new r(i[0],i[1],i[2]);break;case 4:this[he]=new r(i[0],i[1],i[2],i[3]);break;default:throw new Error("Arg list too long.")}},le(J[e],r);const c=new r(function(){});let t;for(t in c)"XMLHttpRequest"===e&&"responseBlob"===t||function(i){"function"==typeof c[i]?J[e].prototype[i]=function(){return this[he][i].apply(this[he],arguments)}:Ne(J[e].prototype,i,{set:function(u){"function"==typeof u?(this[he][i]=Ae(u,e+"."+i),le(this[he][i],u)):this[he][i]=u},get:function(){return this[he][i]}})}(t);for(t in r)"prototype"!==t&&r.hasOwnProperty(t)&&(J[e][t]=r[t])}function se(e,r,c){let t=e;for(;t&&!t.hasOwnProperty(r);)t=Ue(t);!t&&e[r]&&(t=e);const i=U(r);let u=null;if(t&&(!(u=t[i])||!t.hasOwnProperty(i))&&(u=t[i]=t[r],_(t&&De(t,r)))){const p=c(u,i,r);t[r]=function(){return p(this,arguments)},le(t[r],u)}return u}function Ke(e,r,c){let t=null;function i(u){const g=u.data;return g.args[g.cbIdx]=function(){u.invoke.apply(this,arguments)},t.apply(g.target,g.args),u}t=se(e,r,u=>function(g,p){const b=c(g,p);return b.cbIdx>=0&&"function"==typeof p[b.cbIdx]?Fe(b.name,p[b.cbIdx],b,i):u.apply(g,p)})}function le(e,r){e[U("OriginalDelegate")]=r}let We=!1,et=!1;function kt(){if(We)return et;We=!0;try{const e=we.navigator.userAgent;(-1!==e.indexOf("MSIE ")||-1!==e.indexOf("Trident/")||-1!==e.indexOf("Edge/"))&&(et=!0)}catch{}return et}function st(e){return"function"==typeof e}function it(e){return"number"==typeof e}let je=!1;if(typeof window<"u")try{const e=Object.defineProperty({},"passive",{get:function(){je=!0}});window.addEventListener("test",e,e),window.removeEventListener("test",e,e)}catch{je=!1}const bt={useG:!0},de={},ct={},at=new RegExp("^"+Se+"(\\w+)(true|false)$"),lt=U("propagationStopped");function ut(e,r){const c=(r?r(e):e)+_e,t=(r?r(e):e)+pe,i=Se+c,u=Se+t;de[e]={},de[e][_e]=i,de[e][pe]=u}function vt(e,r,c,t){const i=t&&t.add||Ge,u=t&&t.rm||Le,g=t&&t.listeners||"eventListeners",p=t&&t.rmAll||"removeAllListeners",b=U(i),I="."+i+":",E="prependListener",D="."+E+":",H=function(R,d,V){if(R.isRemoved)return;const z=R.callback;let re;"object"==typeof z&&z.handleEvent&&(R.callback=m=>z.handleEvent(m),R.originalDelegate=z);try{R.invoke(R,d,[V])}catch(m){re=m}const W=R.options;return W&&"object"==typeof W&&W.once&&d[u].call(d,V.type,R.originalDelegate?R.originalDelegate:R.callback,W),re};function F(R,d,V){if(!(d=d||e.event))return;const z=R||d.target||e,re=z[de[d.type][V?pe:_e]];if(re){const W=[];if(1===re.length){const m=H(re[0],z,d);m&&W.push(m)}else{const m=re.slice();for(let K=0;K<m.length&&(!d||!0!==d[lt]);K++){const L=H(m[K],z,d);L&&W.push(L)}}if(1===W.length)throw W[0];for(let m=0;m<W.length;m++){const K=W[m];r.nativeScheduleMicroTask(()=>{throw K})}}}const Y=function(R){return F(this,R,!1)},ie=function(R){return F(this,R,!0)};function ce(R,d){if(!R)return!1;let V=!0;d&&void 0!==d.useG&&(V=d.useG);const z=d&&d.vh;let re=!0;d&&void 0!==d.chkDup&&(re=d.chkDup);let W=!1;d&&void 0!==d.rt&&(W=d.rt);let m=R;for(;m&&!m.hasOwnProperty(i);)m=Ue(m);if(!m&&R[i]&&(m=R),!m||m[b])return!1;const K=d&&d.eventNameToString,L={},S=m[b]=m[i],P=m[U(u)]=m[u],N=m[U(g)]=m[g],ue=m[U(p)]=m[p];let Q;d&&d.prepend&&(Q=m[U(d.prepend)]=m[d.prepend]);const ee=V?function(s){if(!L.isExisting)return S.call(L.target,L.eventName,L.capture?ie:Y,L.options)}:function(s){return S.call(L.target,L.eventName,s.invoke,L.options)},B=V?function(s){if(!s.isRemoved){const l=de[s.eventName];let w;l&&(w=l[s.capture?pe:_e]);const O=w&&s.target[w];if(O)for(let y=0;y<O.length;y++)if(O[y]===s){O.splice(y,1),s.isRemoved=!0,s.removeAbortListener&&(s.removeAbortListener(),s.removeAbortListener=null),0===O.length&&(s.allRemoved=!0,s.target[w]=null);break}}if(s.allRemoved)return P.call(s.target,s.eventName,s.capture?ie:Y,s.options)}:function(s){return P.call(s.target,s.eventName,s.invoke,s.options)},be=d&&d.diff?d.diff:function(s,l){const w=typeof l;return"function"===w&&s.callback===l||"object"===w&&s.originalDelegate===l},ve=Zone[U("UNPATCHED_EVENTS")],Te=e[U("PASSIVE_EVENTS")],a=function(s,l,w,O,y=!1,A=!1){return function(){const Z=this||e;let j=arguments[0];d&&d.transferEventName&&(j=d.transferEventName(j));let q=arguments[1];if(!q)return s.apply(this,arguments);if($&&"uncaughtException"===j)return s.apply(this,arguments);let X=!1;if("function"!=typeof q){if(!q.handleEvent)return s.apply(this,arguments);X=!0}if(z&&!z(s,q,Z,arguments))return;const Ce=je&&!!Te&&-1!==Te.indexOf(j),me=function h(s){if("object"==typeof s&&null!==s){const l={...s};return s.signal&&(l.signal=s.signal),l}return s}(function M(s,l){return!je&&"object"==typeof s&&s?!!s.capture:je&&l?"boolean"==typeof s?{capture:s,passive:!0}:s?"object"==typeof s&&!1!==s.passive?{...s,passive:!0}:s:{passive:!0}:s}(arguments[2],Ce)),xe=me?.signal;if(xe?.aborted)return;if(ve)for(let ye=0;ye<ve.length;ye++)if(j===ve[ye])return Ce?s.call(Z,j,q,me):s.apply(this,arguments);const nt=!!me&&("boolean"==typeof me||me.capture),Et=!(!me||"object"!=typeof me)&&me.once,Zt=Zone.current;let rt=de[j];rt||(ut(j,K),rt=de[j]);const Tt=rt[nt?pe:_e];let Qe,Be=Z[Tt],gt=!1;if(Be){if(gt=!0,re)for(let ye=0;ye<Be.length;ye++)if(be(Be[ye],q))return}else Be=Z[Tt]=[];const pt=Z.constructor.name,mt=ct[pt];mt&&(Qe=mt[j]),Qe||(Qe=pt+l+(K?K(j):j)),L.options=me,Et&&(L.options.once=!1),L.target=Z,L.capture=nt,L.eventName=j,L.isExisting=gt;const qe=V?bt:void 0;qe&&(qe.taskData=L),xe&&(L.options.signal=void 0);const ge=Zt.scheduleEventTask(Qe,q,qe,w,O);if(xe){L.options.signal=xe;const ye=()=>ge.zone.cancelTask(ge);s.call(xe,"abort",ye,{once:!0}),ge.removeAbortListener=()=>xe.removeEventListener("abort",ye)}return L.target=null,qe&&(qe.taskData=null),Et&&(L.options.once=!0),!je&&"boolean"==typeof ge.options||(ge.options=me),ge.target=Z,ge.capture=nt,ge.eventName=j,X&&(ge.originalDelegate=q),A?Be.unshift(ge):Be.push(ge),y?Z:void 0}};return m[i]=a(S,I,ee,B,W),Q&&(m[E]=a(Q,D,function(s){return Q.call(L.target,L.eventName,s.invoke,L.options)},B,W,!0)),m[u]=function(){const s=this||e;let l=arguments[0];d&&d.transferEventName&&(l=d.transferEventName(l));const w=arguments[2],O=!!w&&("boolean"==typeof w||w.capture),y=arguments[1];if(!y)return P.apply(this,arguments);if(z&&!z(P,y,s,arguments))return;const A=de[l];let Z;A&&(Z=A[O?pe:_e]);const j=Z&&s[Z];if(j)for(let q=0;q<j.length;q++){const X=j[q];if(be(X,y))return j.splice(q,1),X.isRemoved=!0,0!==j.length||(X.allRemoved=!0,s[Z]=null,O||"string"!=typeof l)||(s[Se+"ON_PROPERTY"+l]=null),X.zone.cancelTask(X),W?s:void 0}return P.apply(this,arguments)},m[g]=function(){const s=this||e;let l=arguments[0];d&&d.transferEventName&&(l=d.transferEventName(l));const w=[],O=ft(s,K?K(l):l);for(let y=0;y<O.length;y++){const A=O[y];w.push(A.originalDelegate?A.originalDelegate:A.callback)}return w},m[p]=function(){const s=this||e;let l=arguments[0];if(l){d&&d.transferEventName&&(l=d.transferEventName(l));const w=de[l];if(w){const A=s[w[_e]],Z=s[w[pe]];if(A){const j=A.slice();for(let q=0;q<j.length;q++){const X=j[q];this[u].call(this,l,X.originalDelegate?X.originalDelegate:X.callback,X.options)}}if(Z){const j=Z.slice();for(let q=0;q<j.length;q++){const X=j[q];this[u].call(this,l,X.originalDelegate?X.originalDelegate:X.callback,X.options)}}}}else{const w=Object.keys(s);for(let O=0;O<w.length;O++){const A=at.exec(w[O]);let Z=A&&A[1];Z&&"removeListener"!==Z&&this[p].call(this,Z)}this[p].call(this,"removeListener")}if(W)return this},le(m[i],S),le(m[u],P),ue&&le(m[p],ue),N&&le(m[g],N),!0}let ne=[];for(let R=0;R<c.length;R++)ne[R]=ce(c[R],t);return ne}function ft(e,r){if(!r){const u=[];for(let g in e){const p=at.exec(g);let b=p&&p[1];if(b&&(!r||b===r)){const I=e[g];if(I)for(let E=0;E<I.length;E++)u.push(I[E])}}return u}let c=de[r];c||(ut(r),c=de[r]);const t=e[c[_e]],i=e[c[pe]];return t?i?t.concat(i):t.slice():i?i.slice():[]}function Rt(e,r){const c=e.Event;c&&c.prototype&&r.patchMethod(c.prototype,"stopImmediatePropagation",t=>function(i,u){i[lt]=!0,t&&t.apply(i,u)})}const Je=U("zoneTask");function He(e,r,c,t){let i=null,u=null;c+=t;const g={};function p(I){const E=I.data;E.args[0]=function(){return I.invoke.apply(this,arguments)};const D=i.apply(e,E.args);return it(D)?E.handleId=D:(E.handle=D,E.isRefreshable=st(D.refresh)),I}function b(I){const{handle:E,handleId:D}=I.data;return u.call(e,E??D)}i=se(e,r+=t,I=>function(E,D){if(st(D[0])){const H={isRefreshable:!1,isPeriodic:"Interval"===t,delay:"Timeout"===t||"Interval"===t?D[1]||0:void 0,args:D},F=D[0];D[0]=function(){try{return F.apply(this,arguments)}finally{const{handle:V,handleId:z,isPeriodic:re,isRefreshable:W}=H;!re&&!W&&(z?delete g[z]:V&&(V[Je]=null))}};const Y=Fe(r,D[0],H,p,b);if(!Y)return Y;const{handleId:ie,handle:ce,isRefreshable:ne,isPeriodic:R}=Y.data;if(ie)g[ie]=Y;else if(ce&&(ce[Je]=Y,ne&&!R)){const d=ce.refresh;ce.refresh=function(){const{zone:V,state:z}=Y;return"notScheduled"===z?(Y._state="scheduled",V._updateTaskCount(Y,1)):"running"===z&&(Y._state="scheduling"),d.call(this)}}return ce??ie??Y}return I.apply(e,D)}),u=se(e,c,I=>function(E,D){const H=D[0];let F;it(H)?(F=g[H],delete g[H]):(F=H?.[Je],F?H[Je]=null:F=H),F?.type?F.cancelFn&&F.zone.cancelTask(F):I.apply(e,D)})}function ht(e,r,c){if(!c||0===c.length)return r;const t=c.filter(u=>u.target===e);if(!t||0===t.length)return r;const i=t[0].ignoreProperties;return r.filter(u=>-1===i.indexOf(u))}function dt(e,r,c,t){e&&Ze(e,ht(e,r,c),t)}function tt(e){return Object.getOwnPropertyNames(e).filter(r=>r.startsWith("on")&&r.length>2).map(r=>r.substring(2))}function Lt(e,r,c,t,i){const u=Zone.__symbol__(t);if(r[u])return;const g=r[u]=r[t];r[t]=function(p,b,I){return b&&b.prototype&&i.forEach(function(E){const D=`${c}.${t}::`+E,H=b.prototype;try{if(H.hasOwnProperty(E)){const F=e.ObjectGetOwnPropertyDescriptor(H,E);F&&F.value?(F.value=e.wrapWithCurrentZone(F.value,D),e._redefineProperty(b.prototype,E,F)):H[E]&&(H[E]=e.wrapWithCurrentZone(H[E],D))}else H[E]&&(H[E]=e.wrapWithCurrentZone(H[E],D))}catch{}}),g.call(r,p,b,I)},e.attachOriginToPatched(r[t],g)}const _t=function Ie(){const e=globalThis,r=!0===e[ae("forceDuplicateZoneCheck")];if(e.Zone&&(r||"function"!=typeof e.Zone.__symbol__))throw new Error("Zone already loaded.");return e.Zone??=function Xe(){const e=fe.performance;function r(M){e&&e.mark&&e.mark(M)}function c(M,T){e&&e.measure&&e.measure(M,T)}r("Zone");let t=(()=>{class M{static{this.__symbol__=ae}static assertZonePatched(){if(fe.Promise!==L.ZoneAwarePromise)throw new Error("Zone.js has detected that ZoneAwarePromise `(window|global).Promise` has been overwritten.\nMost likely cause is that a Promise polyfill has been loaded after Zone.js (Polyfilling Promise api is not necessary when zone.js is loaded. If you must load one, do so before loading zone.js.)")}static get root(){let n=M.current;for(;n.parent;)n=n.parent;return n}static get current(){return P.zone}static get currentTask(){return N}static __load_patch(n,o,v=!1){if(L.hasOwnProperty(n)){const C=!0===fe[ae("forceDuplicateZoneCheck")];if(!v&&C)throw Error("Already loaded patch: "+n)}else if(!fe["__Zone_disable_"+n]){const C="Zone:"+n;r(C),L[n]=o(fe,M,S),c(C,C)}}get parent(){return this._parent}get name(){return this._name}constructor(n,o){this._parent=n,this._name=o?o.name||"unnamed":"<root>",this._properties=o&&o.properties||{},this._zoneDelegate=new u(this,this._parent&&this._parent._zoneDelegate,o)}get(n){const o=this.getZoneWith(n);if(o)return o._properties[n]}getZoneWith(n){let o=this;for(;o;){if(o._properties.hasOwnProperty(n))return o;o=o._parent}return null}fork(n){if(!n)throw new Error("ZoneSpec required!");return this._zoneDelegate.fork(this,n)}wrap(n,o){if("function"!=typeof n)throw new Error("Expecting function got: "+n);const v=this._zoneDelegate.intercept(this,n,o),C=this;return function(){return C.runGuarded(v,this,arguments,o)}}run(n,o,v,C){P={parent:P,zone:this};try{return this._zoneDelegate.invoke(this,n,o,v,C)}finally{P=P.parent}}runGuarded(n,o=null,v,C){P={parent:P,zone:this};try{try{return this._zoneDelegate.invoke(this,n,o,v,C)}catch(ee){if(this._zoneDelegate.handleError(this,ee))throw ee}}finally{P=P.parent}}runTask(n,o,v){if(n.zone!=this)throw new Error("A task can only be run in the zone of creation! (Creation: "+(n.zone||ce).name+"; Execution: "+this.name+")");const C=n,{type:ee,data:{isPeriodic:B=!1,isRefreshable:Pe=!1}={}}=n;if(n.state===ne&&(ee===K||ee===m))return;const be=n.state!=V;be&&C._transitionTo(V,d);const ve=N;N=C,P={parent:P,zone:this};try{ee==m&&n.data&&!B&&!Pe&&(n.cancelFn=void 0);try{return this._zoneDelegate.invokeTask(this,C,o,v)}catch(Te){if(this._zoneDelegate.handleError(this,Te))throw Te}}finally{const Te=n.state;if(Te!==ne&&Te!==re)if(ee==K||B||Pe&&Te===R)be&&C._transitionTo(d,V,R);else{const h=C._zoneDelegates;this._updateTaskCount(C,-1),be&&C._transitionTo(ne,V,ne),Pe&&(C._zoneDelegates=h)}P=P.parent,N=ve}}scheduleTask(n){if(n.zone&&n.zone!==this){let v=this;for(;v;){if(v===n.zone)throw Error(`can not reschedule task to ${this.name} which is descendants of the original zone ${n.zone.name}`);v=v.parent}}n._transitionTo(R,ne);const o=[];n._zoneDelegates=o,n._zone=this;try{n=this._zoneDelegate.scheduleTask(this,n)}catch(v){throw n._transitionTo(re,R,ne),this._zoneDelegate.handleError(this,v),v}return n._zoneDelegates===o&&this._updateTaskCount(n,1),n.state==R&&n._transitionTo(d,R),n}scheduleMicroTask(n,o,v,C){return this.scheduleTask(new g(W,n,o,v,C,void 0))}scheduleMacroTask(n,o,v,C,ee){return this.scheduleTask(new g(m,n,o,v,C,ee))}scheduleEventTask(n,o,v,C,ee){return this.scheduleTask(new g(K,n,o,v,C,ee))}cancelTask(n){if(n.zone!=this)throw new Error("A task can only be cancelled in the zone of creation! (Creation: "+(n.zone||ce).name+"; Execution: "+this.name+")");if(n.state===d||n.state===V){n._transitionTo(z,d,V);try{this._zoneDelegate.cancelTask(this,n)}catch(o){throw n._transitionTo(re,z),this._zoneDelegate.handleError(this,o),o}return this._updateTaskCount(n,-1),n._transitionTo(ne,z),n.runCount=-1,n}}_updateTaskCount(n,o){const v=n._zoneDelegates;-1==o&&(n._zoneDelegates=null);for(let C=0;C<v.length;C++)v[C]._updateTaskCount(n.type,o)}}return M})();const i={name:"",onHasTask:(M,T,n,o)=>M.hasTask(n,o),onScheduleTask:(M,T,n,o)=>M.scheduleTask(n,o),onInvokeTask:(M,T,n,o,v,C)=>M.invokeTask(n,o,v,C),onCancelTask:(M,T,n,o)=>M.cancelTask(n,o)};class u{get zone(){return this._zone}constructor(T,n,o){this._taskCounts={microTask:0,macroTask:0,eventTask:0},this._zone=T,this._parentDelegate=n,this._forkZS=o&&(o&&o.onFork?o:n._forkZS),this._forkDlgt=o&&(o.onFork?n:n._forkDlgt),this._forkCurrZone=o&&(o.onFork?this._zone:n._forkCurrZone),this._interceptZS=o&&(o.onIntercept?o:n._interceptZS),this._interceptDlgt=o&&(o.onIntercept?n:n._interceptDlgt),this._interceptCurrZone=o&&(o.onIntercept?this._zone:n._interceptCurrZone),this._invokeZS=o&&(o.onInvoke?o:n._invokeZS),this._invokeDlgt=o&&(o.onInvoke?n:n._invokeDlgt),this._invokeCurrZone=o&&(o.onInvoke?this._zone:n._invokeCurrZone),this._handleErrorZS=o&&(o.onHandleError?o:n._handleErrorZS),this._handleErrorDlgt=o&&(o.onHandleError?n:n._handleErrorDlgt),this._handleErrorCurrZone=o&&(o.onHandleError?this._zone:n._handleErrorCurrZone),this._scheduleTaskZS=o&&(o.onScheduleTask?o:n._scheduleTaskZS),this._scheduleTaskDlgt=o&&(o.onScheduleTask?n:n._scheduleTaskDlgt),this._scheduleTaskCurrZone=o&&(o.onScheduleTask?this._zone:n._scheduleTaskCurrZone),this._invokeTaskZS=o&&(o.onInvokeTask?o:n._invokeTaskZS),this._invokeTaskDlgt=o&&(o.onInvokeTask?n:n._invokeTaskDlgt),this._invokeTaskCurrZone=o&&(o.onInvokeTask?this._zone:n._invokeTaskCurrZone),this._cancelTaskZS=o&&(o.onCancelTask?o:n._cancelTaskZS),this._cancelTaskDlgt=o&&(o.onCancelTask?n:n._cancelTaskDlgt),this._cancelTaskCurrZone=o&&(o.onCancelTask?this._zone:n._cancelTaskCurrZone),this._hasTaskZS=null,this._hasTaskDlgt=null,this._hasTaskDlgtOwner=null,this._hasTaskCurrZone=null;const v=o&&o.onHasTask;(v||n&&n._hasTaskZS)&&(this._hasTaskZS=v?o:i,this._hasTaskDlgt=n,this._hasTaskDlgtOwner=this,this._hasTaskCurrZone=this._zone,o.onScheduleTask||(this._scheduleTaskZS=i,this._scheduleTaskDlgt=n,this._scheduleTaskCurrZone=this._zone),o.onInvokeTask||(this._invokeTaskZS=i,this._invokeTaskDlgt=n,this._invokeTaskCurrZone=this._zone),o.onCancelTask||(this._cancelTaskZS=i,this._cancelTaskDlgt=n,this._cancelTaskCurrZone=this._zone))}fork(T,n){return this._forkZS?this._forkZS.onFork(this._forkDlgt,this.zone,T,n):new t(T,n)}intercept(T,n,o){return this._interceptZS?this._interceptZS.onIntercept(this._interceptDlgt,this._interceptCurrZone,T,n,o):n}invoke(T,n,o,v,C){return this._invokeZS?this._invokeZS.onInvoke(this._invokeDlgt,this._invokeCurrZone,T,n,o,v,C):n.apply(o,v)}handleError(T,n){return!this._handleErrorZS||this._handleErrorZS.onHandleError(this._handleErrorDlgt,this._handleErrorCurrZone,T,n)}scheduleTask(T,n){let o=n;if(this._scheduleTaskZS)this._hasTaskZS&&o._zoneDelegates.push(this._hasTaskDlgtOwner),o=this._scheduleTaskZS.onScheduleTask(this._scheduleTaskDlgt,this._scheduleTaskCurrZone,T,n),o||(o=n);else if(n.scheduleFn)n.scheduleFn(n);else{if(n.type!=W)throw new Error("Task is missing scheduleFn.");Y(n)}return o}invokeTask(T,n,o,v){return this._invokeTaskZS?this._invokeTaskZS.onInvokeTask(this._invokeTaskDlgt,this._invokeTaskCurrZone,T,n,o,v):n.callback.apply(o,v)}cancelTask(T,n){let o;if(this._cancelTaskZS)o=this._cancelTaskZS.onCancelTask(this._cancelTaskDlgt,this._cancelTaskCurrZone,T,n);else{if(!n.cancelFn)throw Error("Task is not cancelable");o=n.cancelFn(n)}return o}hasTask(T,n){try{this._hasTaskZS&&this._hasTaskZS.onHasTask(this._hasTaskDlgt,this._hasTaskCurrZone,T,n)}catch(o){this.handleError(T,o)}}_updateTaskCount(T,n){const o=this._taskCounts,v=o[T],C=o[T]=v+n;if(C<0)throw new Error("More tasks executed then were scheduled.");0!=v&&0!=C||this.hasTask(this._zone,{microTask:o.microTask>0,macroTask:o.macroTask>0,eventTask:o.eventTask>0,change:T})}}class g{constructor(T,n,o,v,C,ee){if(this._zone=null,this.runCount=0,this._zoneDelegates=null,this._state="notScheduled",this.type=T,this.source=n,this.data=v,this.scheduleFn=C,this.cancelFn=ee,!o)throw new Error("callback is not defined");this.callback=o;const B=this;this.invoke=T===K&&v&&v.useG?g.invokeTask:function(){return g.invokeTask.call(fe,B,this,arguments)}}static invokeTask(T,n,o){T||(T=this),ue++;try{return T.runCount++,T.zone.runTask(T,n,o)}finally{1==ue&&ie(),ue--}}get zone(){return this._zone}get state(){return this._state}cancelScheduleRequest(){this._transitionTo(ne,R)}_transitionTo(T,n,o){if(this._state!==n&&this._state!==o)throw new Error(`${this.type} '${this.source}': can not transition to '${T}', expecting state '${n}'${o?" or '"+o+"'":""}, was '${this._state}'.`);this._state=T,T==ne&&(this._zoneDelegates=null)}toString(){return this.data&&typeof this.data.handleId<"u"?this.data.handleId.toString():Object.prototype.toString.call(this)}toJSON(){return{type:this.type,state:this.state,source:this.source,zone:this.zone.name,runCount:this.runCount}}}const p=ae("setTimeout"),b=ae("Promise"),I=ae("then");let H,E=[],D=!1;function F(M){if(H||fe[b]&&(H=fe[b].resolve(0)),H){let T=H[I];T||(T=H.then),T.call(H,M)}else fe[p](M,0)}function Y(M){0===ue&&0===E.length&&F(ie),M&&E.push(M)}function ie(){if(!D){for(D=!0;E.length;){const M=E;E=[];for(let T=0;T<M.length;T++){const n=M[T];try{n.zone.runTask(n,null,null)}catch(o){S.onUnhandledError(o)}}}S.microtaskDrainDone(),D=!1}}const ce={name:"NO ZONE"},ne="notScheduled",R="scheduling",d="scheduled",V="running",z="canceling",re="unknown",W="microTask",m="macroTask",K="eventTask",L={},S={symbol:ae,currentZoneFrame:()=>P,onUnhandledError:Q,microtaskDrainDone:Q,scheduleMicroTask:Y,showUncaughtError:()=>!t[ae("ignoreConsoleErrorUncaughtError")],patchEventTarget:()=>[],patchOnProperties:Q,patchMethod:()=>Q,bindArguments:()=>[],patchThen:()=>Q,patchMacroTask:()=>Q,patchEventPrototype:()=>Q,isIEOrEdge:()=>!1,getGlobalObjects:()=>{},ObjectDefineProperty:()=>Q,ObjectGetOwnPropertyDescriptor:()=>{},ObjectCreate:()=>{},ArraySlice:()=>[],patchClass:()=>Q,wrapWithCurrentZone:()=>Q,filterProperties:()=>[],attachOriginToPatched:()=>Q,_redefineProperty:()=>Q,patchCallbacks:()=>Q,nativeScheduleMicroTask:F};let P={parent:null,zone:new t(null,null)},N=null,ue=0;function Q(){}return c("Zone","Zone"),t}(),e.Zone}();(function At(e){(function It(e){e.__load_patch("ZoneAwarePromise",(r,c,t)=>{const i=Object.getOwnPropertyDescriptor,u=Object.defineProperty,p=t.symbol,b=[],I=!1!==r[p("DISABLE_WRAPPING_UNCAUGHT_PROMISE_REJECTION")],E=p("Promise"),D=p("then");t.onUnhandledError=h=>{if(t.showUncaughtError()){const a=h&&h.rejection;a?console.error("Unhandled Promise rejection:",a instanceof Error?a.message:a,"; Zone:",h.zone.name,"; Task:",h.task&&h.task.source,"; Value:",a,a instanceof Error?a.stack:void 0):console.error(h)}},t.microtaskDrainDone=()=>{for(;b.length;){const h=b.shift();try{h.zone.runGuarded(()=>{throw h.throwOriginal?h.rejection:h})}catch(a){Y(a)}}};const F=p("unhandledPromiseRejectionHandler");function Y(h){t.onUnhandledError(h);try{const a=c[F];"function"==typeof a&&a.call(this,h)}catch{}}function ie(h){return h&&h.then}function ce(h){return h}function ne(h){return B.reject(h)}const R=p("state"),d=p("value"),V=p("finally"),z=p("parentPromiseValue"),re=p("parentPromiseState"),m=null,K=!0,L=!1;function P(h,a){return s=>{try{M(h,a,s)}catch(l){M(h,!1,l)}}}const N=function(){let h=!1;return function(s){return function(){h||(h=!0,s.apply(null,arguments))}}},ue="Promise resolved with itself",Q=p("currentTaskTrace");function M(h,a,s){const l=N();if(h===s)throw new TypeError(ue);if(h[R]===m){let w=null;try{("object"==typeof s||"function"==typeof s)&&(w=s&&s.then)}catch(O){return l(()=>{M(h,!1,O)})(),h}if(a!==L&&s instanceof B&&s.hasOwnProperty(R)&&s.hasOwnProperty(d)&&s[R]!==m)n(s),M(h,s[R],s[d]);else if(a!==L&&"function"==typeof w)try{w.call(s,l(P(h,a)),l(P(h,!1)))}catch(O){l(()=>{M(h,!1,O)})()}else{h[R]=a;const O=h[d];if(h[d]=s,h[V]===V&&a===K&&(h[R]=h[re],h[d]=h[z]),a===L&&s instanceof Error){const y=c.currentTask&&c.currentTask.data&&c.currentTask.data.__creationTrace__;y&&u(s,Q,{configurable:!0,enumerable:!1,writable:!0,value:y})}for(let y=0;y<O.length;)o(h,O[y++],O[y++],O[y++],O[y++]);if(0==O.length&&a==L){h[R]=0;let y=s;try{throw new Error("Uncaught (in promise): "+function g(h){return h&&h.toString===Object.prototype.toString?(h.constructor&&h.constructor.name||"")+": "+JSON.stringify(h):h?h.toString():Object.prototype.toString.call(h)}(s)+(s&&s.stack?"\n"+s.stack:""))}catch(A){y=A}I&&(y.throwOriginal=!0),y.rejection=s,y.promise=h,y.zone=c.current,y.task=c.currentTask,b.push(y),t.scheduleMicroTask()}}}return h}const T=p("rejectionHandledHandler");function n(h){if(0===h[R]){try{const a=c[T];a&&"function"==typeof a&&a.call(this,{rejection:h[d],promise:h})}catch{}h[R]=L;for(let a=0;a<b.length;a++)h===b[a].promise&&b.splice(a,1)}}function o(h,a,s,l,w){n(h);const O=h[R],y=O?"function"==typeof l?l:ce:"function"==typeof w?w:ne;a.scheduleMicroTask("Promise.then",()=>{try{const A=h[d],Z=!!s&&V===s[V];Z&&(s[z]=A,s[re]=O);const j=a.run(y,void 0,Z&&y!==ne&&y!==ce?[]:[A]);M(s,!0,j)}catch(A){M(s,!1,A)}},s)}const C=function(){},ee=r.AggregateError;class B{static toString(){return"function ZoneAwarePromise() { [native code] }"}static resolve(a){return a instanceof B?a:M(new this(null),K,a)}static reject(a){return M(new this(null),L,a)}static withResolvers(){const a={};return a.promise=new B((s,l)=>{a.resolve=s,a.reject=l}),a}static any(a){if(!a||"function"!=typeof a[Symbol.iterator])return Promise.reject(new ee([],"All promises were rejected"));const s=[];let l=0;try{for(let y of a)l++,s.push(B.resolve(y))}catch{return Promise.reject(new ee([],"All promises were rejected"))}if(0===l)return Promise.reject(new ee([],"All promises were rejected"));let w=!1;const O=[];return new B((y,A)=>{for(let Z=0;Z<s.length;Z++)s[Z].then(j=>{w||(w=!0,y(j))},j=>{O.push(j),l--,0===l&&(w=!0,A(new ee(O,"All promises were rejected")))})})}static race(a){let s,l,w=new this((A,Z)=>{s=A,l=Z});function O(A){s(A)}function y(A){l(A)}for(let A of a)ie(A)||(A=this.resolve(A)),A.then(O,y);return w}static all(a){return B.allWithCallback(a)}static allSettled(a){return(this&&this.prototype instanceof B?this:B).allWithCallback(a,{thenCallback:l=>({status:"fulfilled",value:l}),errorCallback:l=>({status:"rejected",reason:l})})}static allWithCallback(a,s){let l,w,O=new this((j,q)=>{l=j,w=q}),y=2,A=0;const Z=[];for(let j of a){ie(j)||(j=this.resolve(j));const q=A;try{j.then(X=>{Z[q]=s?s.thenCallback(X):X,y--,0===y&&l(Z)},X=>{s?(Z[q]=s.errorCallback(X),y--,0===y&&l(Z)):w(X)})}catch(X){w(X)}y++,A++}return y-=2,0===y&&l(Z),O}constructor(a){const s=this;if(!(s instanceof B))throw new Error("Must be an instanceof Promise.");s[R]=m,s[d]=[];try{const l=N();a&&a(l(P(s,K)),l(P(s,L)))}catch(l){M(s,!1,l)}}get[Symbol.toStringTag](){return"Promise"}get[Symbol.species](){return B}then(a,s){let l=this.constructor?.[Symbol.species];(!l||"function"!=typeof l)&&(l=this.constructor||B);const w=new l(C),O=c.current;return this[R]==m?this[d].push(O,w,a,s):o(this,O,w,a,s),w}catch(a){return this.then(null,a)}finally(a){let s=this.constructor?.[Symbol.species];(!s||"function"!=typeof s)&&(s=B);const l=new s(C);l[V]=V;const w=c.current;return this[R]==m?this[d].push(w,l,a,a):o(this,w,l,a,a),l}}B.resolve=B.resolve,B.reject=B.reject,B.race=B.race,B.all=B.all;const Pe=r[E]=r.Promise;r.Promise=B;const be=p("thenPatched");function ve(h){const a=h.prototype,s=i(a,"then");if(s&&(!1===s.writable||!s.configurable))return;const l=a.then;a[D]=l,h.prototype.then=function(w,O){return new B((A,Z)=>{l.call(this,A,Z)}).then(w,O)},h[be]=!0}return t.patchThen=ve,Pe&&(ve(Pe),se(r,"fetch",h=>function Te(h){return function(a,s){let l=h.apply(a,s);if(l instanceof B)return l;let w=l.constructor;return w[be]||ve(w),l}}(h))),Promise[c.__symbol__("uncaughtPromiseErrors")]=b,B})})(e),function Nt(e){e.__load_patch("toString",r=>{const c=Function.prototype.toString,t=U("OriginalDelegate"),i=U("Promise"),u=U("Error"),g=function(){if("function"==typeof this){const E=this[t];if(E)return"function"==typeof E?c.call(E):Object.prototype.toString.call(E);if(this===Promise){const D=r[i];if(D)return c.call(D)}if(this===Error){const D=r[u];if(D)return c.call(D)}}return c.call(this)};g[t]=c,Function.prototype.toString=g;const p=Object.prototype.toString;Object.prototype.toString=function(){return"function"==typeof Promise&&this instanceof Promise?"[object Promise]":p.call(this)}})}(e),function Mt(e){e.__load_patch("util",(r,c,t)=>{const i=tt(r);t.patchOnProperties=Ze,t.patchMethod=se,t.bindArguments=f,t.patchMacroTask=Ke;const u=c.__symbol__("BLACK_LISTED_EVENTS"),g=c.__symbol__("UNPATCHED_EVENTS");r[g]&&(r[u]=r[g]),r[u]&&(c[u]=c[g]=r[u]),t.patchEventPrototype=Rt,t.patchEventTarget=vt,t.isIEOrEdge=kt,t.ObjectDefineProperty=Ne,t.ObjectGetOwnPropertyDescriptor=De,t.ObjectCreate=$e,t.ArraySlice=Ye,t.patchClass=Oe,t.wrapWithCurrentZone=Ae,t.filterProperties=ht,t.attachOriginToPatched=le,t._redefineProperty=Object.defineProperty,t.patchCallbacks=Lt,t.getGlobalObjects=()=>({globalSources:ct,zoneSymbolEventNames:de,eventNames:i,isBrowser:G,isMix:ke,isNode:$,TRUE_STR:pe,FALSE_STR:_e,ZONE_SYMBOL_PREFIX:Se,ADD_EVENT_LISTENER_STR:Ge,REMOVE_EVENT_LISTENER_STR:Le})})}(e)})(_t),function Ot(e){e.__load_patch("legacy",r=>{const c=r[e.__symbol__("legacyPatch")];c&&c()}),e.__load_patch("timers",r=>{const c="set",t="clear";He(r,c,t,"Timeout"),He(r,c,t,"Interval"),He(r,c,t,"Immediate")}),e.__load_patch("requestAnimationFrame",r=>{He(r,"request","cancel","AnimationFrame"),He(r,"mozRequest","mozCancel","AnimationFrame"),He(r,"webkitRequest","webkitCancel","AnimationFrame")}),e.__load_patch("blocking",(r,c)=>{const t=["alert","prompt","confirm"];for(let i=0;i<t.length;i++)se(r,t[i],(g,p,b)=>function(I,E){return c.current.run(g,r,E,b)})}),e.__load_patch("EventTarget",(r,c,t)=>{(function Dt(e,r){r.patchEventPrototype(e,r)})(r,t),function Ct(e,r){if(Zone[r.symbol("patchEventTarget")])return;const{eventNames:c,zoneSymbolEventNames:t,TRUE_STR:i,FALSE_STR:u,ZONE_SYMBOL_PREFIX:g}=r.getGlobalObjects();for(let b=0;b<c.length;b++){const I=c[b],H=g+(I+u),F=g+(I+i);t[I]={},t[I][u]=H,t[I][i]=F}const p=e.EventTarget;p&&p.prototype&&r.patchEventTarget(e,r,[p&&p.prototype])}(r,t);const i=r.XMLHttpRequestEventTarget;i&&i.prototype&&t.patchEventTarget(r,t,[i.prototype])}),e.__load_patch("MutationObserver",(r,c,t)=>{Oe("MutationObserver"),Oe("WebKitMutationObserver")}),e.__load_patch("IntersectionObserver",(r,c,t)=>{Oe("IntersectionObserver")}),e.__load_patch("FileReader",(r,c,t)=>{Oe("FileReader")}),e.__load_patch("on_property",(r,c,t)=>{!function St(e,r){if($&&!ke||Zone[e.symbol("patchEvents")])return;const c=r.__Zone_ignore_on_properties;let t=[];if(G){const i=window;t=t.concat(["Document","SVGElement","Element","HTMLElement","HTMLBodyElement","HTMLMediaElement","HTMLFrameSetElement","HTMLFrameElement","HTMLIFrameElement","HTMLMarqueeElement","Worker"]);const u=function yt(){try{const e=we.navigator.userAgent;if(-1!==e.indexOf("MSIE ")||-1!==e.indexOf("Trident/"))return!0}catch{}return!1}()?[{target:i,ignoreProperties:["error"]}]:[];dt(i,tt(i),c&&c.concat(u),Ue(i))}t=t.concat(["XMLHttpRequest","XMLHttpRequestEventTarget","IDBIndex","IDBRequest","IDBOpenDBRequest","IDBDatabase","IDBTransaction","IDBCursor","WebSocket"]);for(let i=0;i<t.length;i++){const u=r[t[i]];u&&u.prototype&&dt(u.prototype,tt(u.prototype),c)}}(t,r)}),e.__load_patch("customElements",(r,c,t)=>{!function Pt(e,r){const{isBrowser:c,isMix:t}=r.getGlobalObjects();(c||t)&&e.customElements&&"customElements"in e&&r.patchCallbacks(r,e.customElements,"customElements","define",["connectedCallback","disconnectedCallback","adoptedCallback","attributeChangedCallback","formAssociatedCallback","formDisabledCallback","formResetCallback","formStateRestoreCallback"])}(r,t)}),e.__load_patch("XHR",(r,c)=>{!function I(E){const D=E.XMLHttpRequest;if(!D)return;const H=D.prototype;let Y=H[Me],ie=H[Ve];if(!Y){const S=E.XMLHttpRequestEventTarget;if(S){const P=S.prototype;Y=P[Me],ie=P[Ve]}}const ce="readystatechange",ne="scheduled";function R(S){const P=S.data,N=P.target;N[g]=!1,N[b]=!1;const ue=N[u];Y||(Y=N[Me],ie=N[Ve]),ue&&ie.call(N,ce,ue);const Q=N[u]=()=>{if(N.readyState===N.DONE)if(!P.aborted&&N[g]&&S.state===ne){const T=N[c.__symbol__("loadfalse")];if(0!==N.status&&T&&T.length>0){const n=S.invoke;S.invoke=function(){const o=N[c.__symbol__("loadfalse")];for(let v=0;v<o.length;v++)o[v]===S&&o.splice(v,1);!P.aborted&&S.state===ne&&n.call(S)},T.push(S)}else S.invoke()}else!P.aborted&&!1===N[g]&&(N[b]=!0)};return Y.call(N,ce,Q),N[t]||(N[t]=S),K.apply(N,P.args),N[g]=!0,S}function d(){}function V(S){const P=S.data;return P.aborted=!0,L.apply(P.target,P.args)}const z=se(H,"open",()=>function(S,P){return S[i]=0==P[2],S[p]=P[1],z.apply(S,P)}),W=U("fetchTaskAborting"),m=U("fetchTaskScheduling"),K=se(H,"send",()=>function(S,P){if(!0===c.current[m]||S[i])return K.apply(S,P);{const N={target:S,url:S[p],isPeriodic:!1,args:P,aborted:!1},ue=Fe("XMLHttpRequest.send",d,N,R,V);S&&!0===S[b]&&!N.aborted&&ue.state===ne&&ue.invoke()}}),L=se(H,"abort",()=>function(S,P){const N=function F(S){return S[t]}(S);if(N&&"string"==typeof N.type){if(null==N.cancelFn||N.data&&N.data.aborted)return;N.zone.cancelTask(N)}else if(!0===c.current[W])return L.apply(S,P)})}(r);const t=U("xhrTask"),i=U("xhrSync"),u=U("xhrListener"),g=U("xhrScheduled"),p=U("xhrURL"),b=U("xhrErrorBeforeScheduled")}),e.__load_patch("geolocation",r=>{r.navigator&&r.navigator.geolocation&&function k(e,r){const c=e.constructor.name;for(let t=0;t<r.length;t++){const i=r[t],u=e[i];if(u){if(!_(De(e,i)))continue;e[i]=(p=>{const b=function(){return p.apply(this,f(arguments,c+"."+i))};return le(b,p),b})(u)}}}(r.navigator.geolocation,["getCurrentPosition","watchPosition"])}),e.__load_patch("PromiseRejectionEvent",(r,c)=>{function t(i){return function(u){ft(r,i).forEach(p=>{const b=r.PromiseRejectionEvent;if(b){const I=new b(i,{promise:u.promise,reason:u.rejection});p.invoke(I)}})}}r.PromiseRejectionEvent&&(c[U("unhandledPromiseRejectionHandler")]=t("unhandledrejection"),c[U("rejectionHandledHandler")]=t("rejectionhandled"))}),e.__load_patch("queueMicrotask",(r,c,t)=>{!function wt(e,r){r.patchMethod(e,"queueMicrotask",c=>function(t,i){Zone.current.scheduleMicroTask("queueMicrotask",i[0])})}(r,t)})}(_t)}},fe=>{fe(fe.s=3834)}]);